# -*- coding: utf-8 -*-
"""
Code for Fig2 of:
Tropical mountain ice core 𝜹18O: A Goldilocks indicator for global temperature change

Calculate zonal averaged correlaction/regression between d18Ov and Temp 
using iTRACE data

Created on Mon Sep 27 2021
Last updated on July 21 2023

@author: Yuntao Bao
Contact: bao.291@osu.edu

"""
import numpy as np
import netCDF4 as nc4
import matplotlib.pyplot as plt
import matplotlib as mpl
import statsmodels.api as sm
from scipy.stats import pearsonr
import scipy.signal as signal

def regression(y, x):
    mod_resul = sm.OLS(y, sm.add_constant(x)).fit()
    R, sig = mod_resul.params[1], mod_resul.pvalues[1]    
    return R, sig 

def zscore(x):
    return (x-np.mean(x))/np.std(x)

def Zlev2pressure(x):
    h = x*1000   # input height units: km
    return 100000*(1+(-0.0065/293.5)*(h-0))**(-9.8*0.0289644/(8.31432*(-0.0065)))/100  # hPa

def pressure2Zlev(x):
    press = x*100  # input pressure units: hPa
    return 0+(293.5/(-0.0065))*((press/100000)**(-(8.31432*(-0.0065)/(9.8*0.0289644)))-1)/1000  #km      
   
########################### calculation ###############################  
############# iTRACE deglaciation ##############
season = "ANN"  
case = 'itrace'  
levType =  "presslev" 
time_mod = np.arange(20000-5,11000-5,-10)
var = "T" 
isdetrend = 0

fnamed18Ov = "itrace.20Ka-11Ka.atm."+levType+".d18Ov.filter500."+season+".nc"
fnameT = "itrace.20Ka-11Ka.atm."+levType+".T.filter500."+season+".nc"
fnameQ = "itrace.20Ka-11Ka.atm."+levType+".Q.filter500."+season+".nc"
fpstart = "itrace.20Ka.atm.startlev.ANN.nc"

fnameT_PI = "b.e13.Bi1850C5.f19_g16.05.cam.h0.presslev.1000-1hPa.T.040001-049912.nc"
    
dsd18Ov = nc4.Dataset(fnamed18Ov)
dsT = nc4.Dataset(fnameT)
dsT_PI = nc4.Dataset(fnameT_PI)
dsQ = nc4.Dataset(fnameQ)
dspstart = nc4.Dataset(fpstart)
print(dsT)

lon = dsT['lon'][:].data
lat = dsT['lat'][:].data
if levType == 'presslev':
    lev = dsT['lev_p'][:].data
elif levType == 'sigmalev':    
    lev = dsT['lev_sigma'][:].data
Nx = len(lon)
Ny = len(lat)
Nz = len(lev)
Nyr = 900

d18Ov_data0 = np.nanmean(dsd18Ov["d18Ov"][:].astype('float32'),3)
T_data0 = np.nanmean(dsT["T"][:].astype('float32'),3)-273.15
Q_data0 = np.nanmean(dsQ["Q"][:].astype('float32'),3)*1000  #g/kg 
T_clim_PI = np.nanmean(np.nanmean(dsT_PI["T"][:,10:,:,:].astype('float32'),3),0) - 273.15
T_clim_LGM = np.nanmean(T_data0[:100,:,:],0)

# detrend
if isdetrend == 1:
    d18Ov_data = signal.detrend(d18Ov_data0, axis=0, type='linear').astype('float32')
    T_data = signal.detrend(T_data0, axis=0, type='linear').astype('float32') 
    Q_data = signal.detrend(Q_data0, axis=0, type='linear').astype('float32')     
else:
    d18Ov_data = d18Ov_data0
    T_data = T_data0
    Q_data = Q_data0
    
clevs3 = np.arange(-100, 20.1, 10)
PS_start = np.round(np.nanmean(dspstart["start_lev_idx"][:].astype('float32'),1)).astype('int16')
                
#linear regression 
reg = np.zeros((Nz, Ny), dtype="float32")
reg_p = np.zeros((Nz, Ny), dtype="float32")
corr = np.zeros((Nz, Ny), dtype="float32")
corr_p = np.zeros((Nz, Ny), dtype="float32")
for k in range(Nz):
    for j in range(Ny):
        if levType == 'presslev' and k >= PS_start[j]+1:
            reg[k,j], reg_p[k,j] = np.nan, np.nan
            corr[k,j], corr_p[k,j] = np.nan, np.nan
        else:
            reg[k,j], reg_p[k,j] = regression(d18Ov_data[:,k,j], T_data[:,k,j])
            corr[k,j], corr_p[k,j] = pearsonr(d18Ov_data[:,k,j], T_data[:,k,j])              

corr2 = np.zeros((Nz, Ny), dtype="float32")
corr2_p = np.zeros((Nz, Ny), dtype="float32")
for k in range(Nz):
    for j in range(Ny):
        if levType == 'presslev' and k >= PS_start[j]+1:
            corr2[k,j], corr2_p[k,j] = np.nan, np.nan
        else:
            corr2[k,j], corr2_p[k,j] = pearsonr(T_data[:,k,j], Q_data[:,k,j])

#mask grids with p>0.05 
for j in range(Ny):
    T_clim_PI[PS_start[j]+1:,j] = np.nan
    T_clim_LGM[PS_start[j]+1:,j] = np.nan
    
#%%
############################## plot ##################################
# modify colormap
cmap = plt.get_cmap('coolwarm')
cmapList = [cmap(i) for i in range(cmap.N)]
del (cmapList[int(cmap.N/2)-13:int(cmap.N/2)+13])
cmapList.insert(int(len(cmapList)/2), (1., 1., 1., 1.))
mycmap = cmap.from_list('my_cmap', cmapList[0:], cmap.N)

clevsTslp = np.array([-1.8,-1.4,-1.,-0.6,-0.2,-0.05,0.05,0.2,0.6,1.,1.4,1.8])
clevsT = np.arange(-80, 30+0.1, 10)
clevs1 = np.array([-0.9,-0.7,-0.5,-0.3,-0.1,0.1, 0.3,0.5,0.7,0.9])
clevs3 = np.arange(-70, 20.1, 10)
   
[Y, Z] = np.meshgrid(lat, lev)

fig = plt.figure(figsize=(8,10))
#axes1 = fig.add_subplot(2,1,1)
axes1 = fig.add_axes([0.15, 0.54, 0.64, 0.34])
axes1.set_title('(a) Corr: <d18Ov,'+var+'>',loc='center', fontweight='bold')
axes1.invert_yaxis() 
c1 = axes1.contourf(Y, Z, corr, 
                   levels=clevs1, extend='both',cmap=mycmap)  
c11 = axes1.contour(Y, Z, T_clim_PI, 
                   levels=clevs3, colors='black')  
c111 = axes1.contour(Y, Z, T_clim_PI, levels=0, linestyles='dashdot', linewidths=3,
                     colors='black') 
c1111 = axes1.contour(Y, Z, T_clim_LGM, levels=0, linestyles='dashdot', linewidths=3,
                     colors='green')    
axes1.clabel(c11, inline=True, fontsize=10, fmt='%d')
axes1.scatter(-9, Zlev2pressure(6050/1000), 30, color='yellow')
#axes1.scatter(-18, Zlev2pressure(6540/1000), 30, color='lime') 
axes1.text(-7, Zlev2pressure(6050/1000-0.4), 'Huascarán', color='yellow', fontweight='bold')  
#axes1.text(-16, Zlev2pressure(6540/1000+0.2), 'Sajama', color='lime', fontweight='bold')    
axes1.set_ylabel('Pressure (hPa)', fontweight='bold')
axt1 = axes1.secondary_yaxis('right', functions=(pressure2Zlev, Zlev2pressure))   
axt1.set_ylim([0,15])
axt1.set_yticks([0,1,2,3,4,6,8,10,15])
axt1.set_ylabel('Level (km)', fontweight='bold')  
axes1.set_facecolor('gray')
axes1.set_xticklabels('')
lbposit1=fig.add_axes([0.87, 0.55, 0.015, 0.3])
fig.colorbar(c1, cax = lbposit1, orientation='vertical',format='%.1f')

axes2 = fig.add_axes([0.15, 0.13, 0.64, 0.34])
#axes2 = fig.add_subplot(2,1,2)
axes2.set_title('(b) Reg: <d18Ov,'+var+'>',loc='center', fontweight='bold')
axes2.invert_yaxis() 
c2 = axes2.contourf(Y, Z, reg, 
                   levels=clevsTslp, extend='both',cmap=mycmap)                   
c22 = axes2.contour(Y, Z, T_clim_PI, 
                   levels=clevs3, colors='black') 
c222 = axes2.contour(Y, Z, T_clim_PI, levels=0, linestyles='dashdot', linewidths=3,
                     colors='black')   
c2222 = axes2.contour(Y, Z, T_clim_LGM, levels=0, linestyles='dashdot', linewidths=3,
                     colors='green')  
axes2.clabel(c22, inline=True, fontsize=10, fmt='%d')
axes2.scatter(-9, Zlev2pressure(6050/1000), 30, color='yellow')
#axes2.scatter(-18, Zlev2pressure(6540/1000), 30, color='lime') 
axes2.text(-7, Zlev2pressure(6050/1000-0.4), 'Huascarán', color='yellow', fontweight='bold')  
#axes2.text(-16, Zlev2pressure(6540/1000+0.2), 'Sajama', color='lime', fontweight='bold') 
axes2.set_ylabel('Pressure (hPa)', fontweight='bold')
axt2 = axes2.secondary_yaxis('right', functions=(pressure2Zlev, Zlev2pressure))   
axt2.set_ylim([0,15])
axt2.set_yticks([0,1,2,3,4,6,8,10,15])
axt2.set_ylabel('Level (km)', fontweight='bold')  
axes2.set_facecolor('gray')
lbposit2=fig.add_axes([0.87, 0.15, 0.015, 0.3])
fig.colorbar(c2, cax = lbposit2, orientation='vertical',format='%.2f')
axes2.set_xlabel('lat', fontweight='bold')
